#!/usr/bin/env python
# coding: utf-8

import os
import mimetypes
import json
import logging
import datetime
import pytz
from os.path import splitext, basename, abspath
from tzlocal import get_localzone

from configparser import RawConfigParser
from requests_oauthlib import OAuth2Session
from oauthlib.oauth2 import LegacyApplicationClient
from requests_toolbelt.multipart.encoder import MultipartEncoder

from . import utils

PEERTUBE_SECRETS_FILE = 'peertube_secret'
PEERTUBE_PRIVACY = {
    "public": 1,
    "unlisted": 2,
    "private": 3
}


def get_authenticated_service(secret):
    peertube_url = str(secret.get('peertube', 'peertube_url')).rstrip("/")

    oauth_client = LegacyApplicationClient(
        client_id=str(secret.get('peertube', 'client_id'))
    )
    try:
        oauth = OAuth2Session(client=oauth_client)
        oauth.fetch_token(
            token_url=str(peertube_url + '/api/v1/users/token'),
            # lower as peertube does not store uppercase for pseudo
            username=str(secret.get('peertube', 'username').lower()),
            password=str(secret.get('peertube', 'password')),
            client_id=str(secret.get('peertube', 'client_id')),
            client_secret=str(secret.get('peertube', 'client_secret'))
        )
    except Exception as e:
        if hasattr(e, 'message'):
            logging.error("Peertube: Error: " + str(e.message))
            exit(1)
        else:
            logging.error("Peertube: Error: " + str(e))
            exit(1)
    return oauth


def get_default_channel(user_info):
    return user_info['videoChannels'][0]['id']


def get_channel_by_name(user_info, options):
    for channel in user_info["videoChannels"]:
        if channel['displayName'] == options.get('--channel'):
            return channel['id']


def create_channel(oauth, url, options):
    template = ('Peertube: Channel %s does not exist, creating it.')
    logging.info(template % (str(options.get('--channel'))))
    channel_name = utils.cleanString(str(options.get('--channel')))
    # Peertube allows 20 chars max for channel name
    channel_name = channel_name[:19]
    data = '{"name":"' + channel_name + '", \
            "displayName":"' + options.get('--channel') + '", \
            "description":null, \
            "support":null}'

    headers = {
        'Content-Type': "application/json; charset=UTF-8"
    }
    try:
        response = oauth.post(url + "/api/v1/video-channels/",
                              data=data.encode('utf-8'),
                              headers=headers)
    except Exception as e:
        if hasattr(e, 'message'):
            logging.error("Error: " + str(e.message))
        else:
            logging.error("Error: " + str(e))
    if response is not None:
        if response.status_code == 200:
            jresponse = response.json()
            jresponse = jresponse['videoChannel']
            return jresponse['id']
        if response.status_code == 409:
            logging.error('Peertube: Error: It seems there is a conflict with an existing channel named '
                          + channel_name + '.'
                          ' Please beware Peertube internal name is compiled from 20 firsts characters of channel name.'
                          ' Also note that channel name are not case sensitive (no uppercase nor accent)'
                          ' Please check your channel name and retry.')
            exit(1)
        else:
            logging.error(('Peertube: Creating channel failed with an unexpected response: '
                           '%s') % response)
            exit(1)


def get_default_playlist(user_info):
    return user_info['videoChannels'][0]['id']


def get_playlist_by_name(user_playlists, options):
    for playlist in user_playlists["data"]:
        if playlist['displayName'] == options.get('--playlist'):
            return playlist['id']


def create_playlist(oauth, url, options, channel):
    template = ('Peertube: Playlist %s does not exist, creating it.')
    logging.info(template % (str(options.get('--playlist'))))
    # We use files for form-data Content
    # see https://requests.readthedocs.io/en/latest/user/quickstart/#post-a-multipart-encoded-file
    # None is used to mute "filename" field
    files = {'displayName': (None, str(options.get('--playlist'))),
             'privacy': (None, "1"),
             'description': (None, "null"),
             'videoChannelId': (None, str(channel)),
             'thumbnailfile': (None, "null")}
    try:
        response = oauth.post(url + "/api/v1/video-playlists/",
                       files=files)
    except Exception as e:
        if hasattr(e, 'message'):
            logging.error("Error: " + str(e.message))
        else:
            logging.error("Error: " + str(e))
    if response is not None:
        if response.status_code == 200:
            jresponse = response.json()
            jresponse = jresponse['videoPlaylist']
            return jresponse['id']
        else:
            logging.error(('Peertube: Creating the playlist failed with an unexpected response: '
                           '%s') % response)
            exit(1)


def set_playlist(oauth, url, video_id, playlist_id):
    logging.info('Peertube: add video to playlist.')
    data = '{"videoId":"' + str(video_id) + '"}'

    headers = {
        'Content-Type': "application/json"
    }
    try:
        response = oauth.post(url + "/api/v1/video-playlists/"+str(playlist_id)+"/videos",
                              data=data,
                              headers=headers)
    except Exception as e:
        if hasattr(e, 'message'):
            logging.error("Error: " + str(e.message))
        else:
            logging.error("Error: " + str(e))
    if response is not None:
        if response.status_code == 200:
            logging.info('Peertube: Video is successfully added to the playlist.')
        else:
            logging.error(('Peertube: Configuring the playlist failed with an unexpected response: '
                           '%s') % response)
            exit(1)


def upload_video(oauth, secret, options):

    def get_userinfo():
        return json.loads(oauth.get(url+"/api/v1/users/me").content)

    def get_file(path):
        mimetypes.init()
        return (basename(path), open(abspath(path), 'rb'),
                mimetypes.types_map[splitext(path)[1]])

    def get_playlist(username):
        return json.loads(oauth.get(url+"/api/v1/accounts/"+username+"/video-playlists").content)

    path = options.get('--file')
    url = str(secret.get('peertube', 'peertube_url')).rstrip('/')
    user_info = get_userinfo()
    user_playlists = get_playlist(str(secret.get('peertube', 'username').lower()))

    # We need to transform fields into tuple to deal with tags as
    # MultipartEncoder does not support list refer
    # https://github.com/requests/toolbelt/issues/190 and
    # https://github.com/requests/toolbelt/issues/205
    fields = [
        ("name", options.get('--name') or splitext(basename(options.get('--file')))[0]),
        ("licence", "1"),
        ("description", options.get('--description')  or "default description"),
        ("nsfw", str(int(options.get('--nsfw')) or "0")),
        ("videofile", get_file(path))
    ]

    if options.get('--tags'):
        tags = options.get('--tags').split(',')
        for strtag in tags:
            # Empty tag crashes Peertube, so skip them
            if strtag == "":
                continue
            # Tag more than 30 chars crashes Peertube, so exit and check tags
            if len(strtag) >= 30:
                logging.warning("Peertube: Sorry, Peertube does not support tag with more than 30 characters, please reduce tag: " + strtag)
                exit(1)
            fields.append(("tags[]", strtag))

    if options.get('--category'):
        fields.append(("category", str(utils.getCategory(options.get('--category'), 'peertube'))))
    else:
        # if no category, set default to 2 (Films)
        fields.append(("category", "2"))

    if options.get('--language'):
        fields.append(("language", str(utils.getLanguage(options.get('--language'), "peertube"))))
    else:
        # if no language, set default to 1 (English)
        fields.append(("language", "en"))

    if options.get('--disable-comments'):
        fields.append(("commentsEnabled", "0"))
    else:
        fields.append(("commentsEnabled", "1"))

    privacy = None
    if options.get('--privacy'):
        privacy = options.get('--privacy').lower()

    # If peertubeAt exists, use instead of publishAt
    if options.get('--peertubeAt'):
        publishAt = options.get('--peertubeAt')
    elif options.get('--publishAt'):
        publishAt = options.get('--publishAt')

    if 'publishAt' in locals():
        publishAt = datetime.datetime.strptime(publishAt, '%Y-%m-%dT%H:%M:%S')
        tz = get_localzone()
        tz = pytz.timezone(str(tz))
        publishAt = tz.localize(publishAt).isoformat()
        fields.append(("scheduleUpdate[updateAt]", publishAt))
        fields.append(("scheduleUpdate[privacy]", str(PEERTUBE_PRIVACY["public"])))
        fields.append(("privacy", str(PEERTUBE_PRIVACY["private"])))
    else:
        fields.append(("privacy", str(PEERTUBE_PRIVACY[privacy or "private"])))

    if options.get('--thumbnail'):
        fields.append(("thumbnailfile", get_file(options.get('--thumbnail'))))
        fields.append(("previewfile", get_file(options.get('--thumbnail'))))

    if options.get('--channel'):
        channel_id = get_channel_by_name(user_info, options)
        if not channel_id and options.get('--channelCreate'):
            channel_id = create_channel(oauth, url, options)
        elif not channel_id:
            logging.warning("Channel `" + options.get('--channel') + "` is unknown, using default channel.")
            channel_id = get_default_channel(user_info)
    else:
        channel_id = get_default_channel(user_info)

    fields.append(("channelId", str(channel_id)))

    if options.get('--playlist'):
        playlist_id = get_playlist_by_name(user_playlists, options)
        if not playlist_id and options.get('--playlistCreate'):
            playlist_id = create_playlist(oauth, url, options, channel_id)
        elif not playlist_id:
            logging.warning("Playlist `" + options.get('--playlist') + "` does not exist, please set --playlistCreate"
                            " if you want to create it")
            exit(1)

    multipart_data = MultipartEncoder(fields)

    headers = {
        'Content-Type': multipart_data.content_type
    }
    response = oauth.post(url + "/api/v1/videos/upload",
                          data=multipart_data,
                          headers=headers)
    if response is not None:
        if response.status_code == 200:
            jresponse = response.json()
            jresponse = jresponse['video']
            uuid = jresponse['uuid']
            video_id = str(jresponse['id'])
            logging.info('Peertube : Video was successfully uploaded.')
            template = 'Peertube: Watch it at %s/videos/watch/%s.'
            logging.info(template % (url, uuid))
            # Upload is successful we may set playlist
            if options.get('--playlist'):
                set_playlist(oauth, url, video_id, playlist_id)
        else:
            logging.error(('Peertube: The upload failed with an unexpected response: '
                           '%s') % response)
            exit(1)


def run(options):
    secret = RawConfigParser()
    try:
        secret.read(PEERTUBE_SECRETS_FILE)
    except Exception as e:
        logging.error("Peertube: Error loading " + str(PEERTUBE_SECRETS_FILE) + ": " + str(e))
        exit(1)
    insecure_transport = secret.get('peertube', 'OAUTHLIB_INSECURE_TRANSPORT')
    os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = insecure_transport
    oauth = get_authenticated_service(secret)
    try:
        logging.info('Peertube: Uploading video...')
        upload_video(oauth, secret, options)
    except Exception as e:
        if hasattr(e, 'message'):
            logging.error("Peertube: Error: " + str(e.message))
        else:
            logging.error("Peertube: Error: " + str(e))



#!/usr/bin/python
# coding: utf-8

from configparser import RawConfigParser, NoOptionError, NoSectionError
from os.path import dirname, splitext, basename, isfile
import re
from os import devnull
from subprocess import check_call, CalledProcessError, STDOUT
import unidecode
import logging

### CATEGORIES ###
YOUTUBE_CATEGORY = {
    "music": 10,
    "films": 1,
    "vehicles": 2,
    "sport": 17,
    "travels": 19,
    "gaming": 20,
    "people": 22,
    "comedy": 23,
    "entertainment": 24,
    "news": 25,
    "how to": 26,
    "education": 27,
    "activism": 29,
    "science & technology": 28,
    "science": 28,
    "technology": 28,
    "animals": 15
}

PEERTUBE_CATEGORY = {
    "music": 1,
    "films": 2,
    "vehicles": 3,
    "sport": 5,
    "travels": 6,
    "gaming": 7,
    "people": 8,
    "comedy": 9,
    "entertainment": 10,
    "news": 11,
    "how to": 12,
    "education": 13,
    "activism": 14,
    "science & technology": 15,
    "science": 15,
    "technology": 15,
    "animals": 16
}

### LANGUAGES ###
YOUTUBE_LANGUAGE = {
    "arabic": 'ar',
    "english": 'en',
    "french": 'fr',
    "german": 'de',
    "hindi": 'hi',
    "italian": 'it',
    "japanese": 'ja',
    "korean": 'ko',
    "mandarin": 'zh-CN',
    "portuguese": 'pt-PT',
    "punjabi": 'pa',
    "russian": 'ru',
    "spanish": 'es'
}

PEERTUBE_LANGUAGE = {
    "arabic": "ar",
    "english": "en",
    "french": "fr",
    "german": "de",
    "hindi": "hi",
    "italian": "it",
    "japanese": "ja",
    "korean": "ko",
    "mandarin": "zh",
    "portuguese": "pt",
    "punjabi": "pa",
    "russian": "ru",
    "spanish": "es"
}
######################


def getCategory(category, platform):
    if platform == "youtube":
        return YOUTUBE_CATEGORY[category.lower()]
    else:
        return PEERTUBE_CATEGORY[category.lower()]


def getLanguage(language, platform):
    if platform == "youtube":
        return YOUTUBE_LANGUAGE[language.lower()]
    else:
        return PEERTUBE_LANGUAGE[language.lower()]


def remove_empty_kwargs(**kwargs):
    good_kwargs = {}
    if kwargs is not None:
        for key, value in kwargs.items():
            if value:
                good_kwargs[key] = value
    return good_kwargs


def searchThumbnail(options):
    video_directory = dirname(options.get('--file')) + "/"
    # First, check for thumbnail based on videoname
    if options.get('--name'):
        if isfile(video_directory + options.get('--name') + ".jpg"):
            options['--thumbnail'] = video_directory + options.get('--name') + ".jpg"
        elif isfile(video_directory + options.get('--name') + ".jpeg"):
            options['--thumbnail'] = video_directory + options.get('--name') + ".jpeg"
    # Then, if we still not have thumbnail, check for thumbnail based on videofile name
    if not options.get('--thumbnail'):
        video_file = splitext(basename(options.get('--file')))[0]
        if isfile(video_directory + video_file + ".jpg"):
            options['--thumbnail'] = video_directory + video_file + ".jpg"
        elif isfile(video_directory + video_file + ".jpeg"):
            options['--thumbnail'] = video_directory + video_file + ".jpeg"
    return options


# return the nfo as a RawConfigParser object
def loadNFO(filename):
    try:
        logging.info("Loading " + filename + " as NFO")
        nfo = RawConfigParser()
        nfo.read(filename, encoding='utf-8')
        return nfo
    except Exception as e:
        logging.error("Problem loading NFO file " + filename + ": " + str(e))
        exit(1)
    return False


def parseNFO(options):
    video_directory = dirname(options.get('--file'))
    directory_name = basename(video_directory)
    nfo_txt = False
    nfo_directory = False
    nfo_videoname = False
    nfo_file = False
    nfo_cli = False

    if isfile(video_directory + "/" + "nfo.txt"):
        nfo_txt = loadNFO(video_directory + "/" + "nfo.txt")
    elif isfile(video_directory + "/" + "NFO.txt"):
        nfo_txt = loadNFO(video_directory + "/" + "NFO.txt")

    if isfile(video_directory + "/" + directory_name+ ".txt"):
        nfo_directory = loadNFO(video_directory + "/" + directory_name+ ".txt")

    if options.get('--name'):
        if isfile(video_directory + "/" + options.get('--name')):
            nfo_videoname = loadNFO(video_directory + "/" + options.get('--name') + ".txt")

    video_file = splitext(basename(options.get('--file')))[0]
    if isfile(video_directory + "/" + video_file + ".txt"):
        nfo_file = loadNFO(video_directory + "/" + video_file + ".txt")

    if options.get('--nfo'):
        if isfile(options.get('--nfo')):
            nfo_cli = loadNFO(options.get('--nfo'))
        else:
            logging.error("Given NFO file does not exist, please check your path.")
            exit(1)

    # We need to load NFO in this exact order to keep the priorities
    # options in cli > nfo_cli > nfo_file > nfo_videoname > nfo_directory > nfo_txt
    for nfo in [nfo_cli, nfo_file, nfo_videoname, nfo_directory, nfo_txt]:
        if nfo:
            # We need to check all options and replace it with the nfo value if not defined (None or False)
            for key, value in options.items():
                key = key.replace("-", "")
                try:
                    # get string options
                    if value is None and nfo.get('video', key):
                        options['--' + key] = nfo.get('video', key)
                    # get boolean options
                    elif value is False and nfo.getboolean('video', key):
                        options['--' + key] = nfo.getboolean('video', key)
                except NoOptionError:
                    continue
                except NoSectionError:
                    logging.error(nfo + " misses section [video], please check syntax of your NFO.")
                    exit(1)
    return options


def upcaseFirstLetter(s):
    return s[0].upper() + s[1:]


def cleanString(toclean):
    toclean = unidecode.unidecode(toclean)
    cleaned = re.sub('[^A-Za-z0-9]+', '', toclean)

    return cleaned



from os.path import join, abspath, isfile, dirname
from os import listdir
from shutil import copyfile


def genconfig():
    path = join(dirname(__file__), 'config')
    files = [f for f in listdir(path) if isfile(join(path, f))]

    for f in files:
        copyfile(join(path, f), f)


if __name__ == '__main__':
    genconfig()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['prismedia']

package_data = \
{'': ['*'],
 'prismedia': ['config/*',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt']}

install_requires = \
['Unidecode>=1.0.23,<2.0.0',
 'configparser>=3.7.1,<4.0.0',
 'docopt>=0.6.2,<0.7.0',
 'future>=0.17.1,<0.18.0',
 'google-api-python-client>=1.7.6,<2.0.0',
 'google-auth-httplib2>=0.0.3,<0.0.4',
 'google-auth-oauthlib>=0.2.0,<0.3.0',
 'google-auth>=1.6.1,<2.0.0',
 'httplib2>=0.12.1,<0.13.0',
 'oauthlib>=2.1.0,<3.0.0',
 'python-magic>=0.4.15,<0.5.0',
 'requests-oauthlib>=0.8.0,<0.9.0',
 'requests-toolbelt>=0.9.1,<0.10.0',
 'requests>=2.18.4,<3.0.0',
 'schema>=0.6.8,<0.7.0',
 'tzlocal>=1.5.1,<2.0.0',
 'uritemplate>=3.0.0,<4.0.0',
 'urllib3>=1.22,<2.0']

extras_require = \
{':platform_system == "Windows"': ['python-magic-bin>=0.4.14,<0.5.0']}

entry_points = \
{'console_scripts': ['prismedia = prismedia.upload:main']}

setup_kwargs = {
    'name': 'prismedia',
    'version': '0.9.1',
    'description': 'scripting your way to upload videos on peertube and youtube',
    'long_description': '# Prismedia\n\nScripting your way to upload videos to peertube and youtube. Works with Python 3.5+.\n\n[TOC]: #\n\n## Table of Contents\n- [Installation](#installation)\n  - [From pip](#from-pip)\n  - [From source](#from-source)\n- [Configuration](#configuration)\n  - [Peertube](#peertube)\n  - [Youtube](#youtube)\n- [Usage](#usage)\n- [Enhanced use of NFO](#enhanced-use-of-nfo)\n- [Features](#features)\n- [Compatibility](#compatibility)\n- [Sources](#sources)\n- [Contributors](#contributors)\n\n## Installation an upgrade\n\n### From pip\n\nSimply install with \n\n```bash\npip install prismedia\n```\n\nUpgrade with \n\n```bash\npip install --upgrade prismedia\n```\n\n### From source\n\nGet the source: \n\n```bash\ngit clone https://git.lecygnenoir.info/LecygneNoir/prismedia.git prismedia\n```\n\nYou may use pip to install requirements: `pip install -r requirements.txt` if you want to use the script directly.  \n(*note:* requirements are generated via `poetry export -f requirements.txt`)\n\nOtherwise, you can use [poetry](https://python-poetry.org), which create a virtualenv for the project directly  \n(Or use the existing virtualenv if one is activated)\n\n```\npoetry install\n```\n\n\n## Configuration\n\nGenerate sample files with `python -m prismedia.genconfig`.  \nThen rename and edit `peertube_secret` and `youtube_secret.json` with your credentials. (see below)\n\n### Peertube\nSet your credentials, peertube server URL.  \nYou can get client_id and client_secret by logging in your peertube website and reaching the URL:  \nhttps://domain.example/api/v1/oauth-clients/local  \nYou can set ``OAUTHLIB_INSECURE_TRANSPORT`` to 1 if you do not use https (not recommended)\n\n### Youtube\nYoutube uses combination of oauth and API access to identify.\n\n**Credentials**\nThe first time you connect, prismedia will open your browser to ask you to authenticate to\nYoutube and allow the app to use your Youtube channel.  \n**It is here you choose which channel you will upload to**.  \nOnce authenticated, the token is stored inside the file ``.youtube_credentials.json``.  \nPrismedia will try to use this file at each launch, and re-ask for authentication if it does not exist.\n\n**Oauth**:  \nThe default youtube_secret.json should allow you to upload some videos.  \nIf you plan a larger usage, please consider creating your own youtube_secret file:\n\n - Go to the [Google console](https://console.developers.google.com/).\n - Create project.\n - Side menu: APIs & auth -> APIs\n - Top menu: Enabled API(s): Enable all Youtube APIs.\n - Side menu: APIs & auth -> Credentials.\n - Create a Client ID: Add credentials -> OAuth 2.0 Client ID -> Other -> Name: prismedia1 -> Create -> OK\n - Download JSON: Under the section "OAuth 2.0 client IDs". Save the file to your local system.\n - Save this JSON as your youtube_secret.json file.\n\n## Usage\nSupport only mp4 for cross compatibility between Youtube and Peertube.  \n**Note that all options may be specified in a NFO file!** (see [Enhanced NFO](#enhanced-use-of-nfo))\n\nUpload a video:\n\n```\nprismedia --file="yourvideo.mp4"\n```\n\nSpecify description and tags:\n\n```\nprismedia --file="yourvideo.mp4" -d "My supa description" -t "tag1,tag2,foo"\n```\n\nProvide a thumbnail:\n\n```\nprismedia --file="yourvideo.mp4" -d "Video with thumbnail" --thumbnail="/path/to/your/thumbnail.jpg"\n```\n\n\nUse a NFO file to specify your video options:  \n(See [Enhanced NFO](#enhanced-use-of-nfo) for more precise example)\n```\nprismedia --file="yourvideo.mp4" --nfo /path/to/your/nfo.txt\n```\n\n\nUse --help to get all available options:\n\n```\nOptions:\n  -f, --file=STRING Path to the video file to upload in mp4\n  --name=NAME  Name of the video to upload. (default to video filename)\n  --debug  Trigger some debug information like options used (default: no)\n  -d, --description=STRING  Description of the video. (default: default description)\n  -t, --tags=STRING  Tags for the video. comma separated.\n                     WARN: tags with punctuation (!, \', ", ?, ...)\n                           are not supported by Mastodon to be published from Peertube\n  -c, --category=STRING  Category for the videos, see below. (default: Films)\n  --cca  License should be CreativeCommon Attribution (affects Youtube upload only)\n  -p, --privacy=STRING  Choose between public, unlisted or private. (default: private)\n  --disable-comments  Disable comments (Peertube only as YT API does not support) (default: comments are enabled)\n  --nsfw  Set the video as No Safe For Work (Peertube only as YT API does not support) (default: video is safe)\n  --nfo=STRING  Configure a specific nfo file to set options for the video.\n                By default Prismedia search a .txt based on the video name and will\n                decode the file as UTF-8 (so make sure your nfo file is UTF-8 encoded)\n                See nfo_example.txt for more details\n  --platform=STRING  List of platform(s) to upload to, comma separated.\n                     Supported platforms are youtube and peertube (default is both)\n  --language=STRING  Specify the default language for video. See below for supported language. (default is English)\n  --publishAt=DATE  Publish the video at the given DATE using local server timezone.\n                    DATE should be on the form YYYY-MM-DDThh:mm:ss eg: 2018-03-12T19:00:00\n                    DATE should be in the future\n  --peertubeAt=DATE\n  --youtubeAt=DATE  Override publishAt for the corresponding platform. Allow to create preview on specific platform\n  --thumbnail=STRING    Path to a file to use as a thumbnail for the video.\n                        Supported types are jpg and jpeg.\n                        By default, prismedia search for an image based on video name followed by .jpg or .jpeg\n  --channel=STRING Set the channel to use for the video (Peertube only)\n                    If the channel is not found, spawn an error except if --channelCreate is set.\n  --channelCreate  Create the channel if not exists. (Peertube only, default do not create)\n                   Only relevant if --channel is set.\n  --playlist=STRING Set the playlist to use for the video.\n                    If the playlist is not found, spawn an error except if --playlistCreate is set.\n  --playlistCreate  Create the playlist if not exists. (default do not create)\n                    Only relevant if --playlist is set.\n  -h --help  Show this help.\n  --version  Show version.\n\nCategories:\n  Category is the type of video you upload. Default is films.\n  Here are available categories from Peertube and Youtube:\n    music, films, vehicles,\n    sports, travels, gaming, people,\n    comedy, entertainment, news,\n    how to, education, activism, science & technology,\n    science, technology, animals\n\nLanguages:\n  Language of the video (audio track), choose one. Default is English\n  Here are available languages from Peertube and Youtube:\n    Arabic, English, French, German, Hindi, Italian,\n    Japanese, Korean, Mandarin, Portuguese, Punjabi, Russian, Spanish\n```\n\n## Enhanced use of NFO\nSince Prismedia v0.9.0, the NFO system has been improved to allow hierarchical loading.\nFirst of all, **if you already used nfo**, either with `--nfo` or by using `videoname.txt`, nothing changes :-)\n\nBut you are now able to use a more flexible NFO system, by using priorities. This allow you to set some defaults to avoid recreating a full nfo for each video\n\nBasically, Prismedia will now load options in this order, using the last value found in case of conflict:  \n`nfo.txt < directory_name.txt < video_name.txt < command line NFO < command line argument`\n\nYou\'ll find a complete set of samples in the [prismedia/samples](prismedia/samples) directory so let\'s take it as an example:\n```\n$ tree Recipes/\nRecipes/\n├── cli_nfo.txt\n├── nfo.txt\n├── samples.txt\n├── yourvideo1.mp4\n├── yourvideo1.txt\n├── yourvideo1.jpg\n├── yourvideo2.mp4\n└── yourvideo2.txt\n```\n\nBy using \n```\nprismedia --file=/path/to/Recipes/yourvideo1.mp4 --nfo=/path/to/Recipes/cli_nfo.txt --cca\n```\n\nPrismedia will:\n- look for options in `nfo.txt`\n- look for options in `samples.txt` (from directory name) and erase any previous conflicting options\n- look for options in `yourvideo1.txt` (from video name) and erase any previous conflicting options\n- look for options in `cli_nfo.txt` (from the `--nfo` in command line) and erase any previous conflicting options\n- erase any previous option regarding CCA as it\'s specified in cli with `--cca`\n- take `yourvideo1.jpg` as thumbnail if no other files has been specified in previous NFO\n\nIn other word, Prismedia will now use option given in cli, then look for option in cli_nfo.txt, then complete with video_name.txt, then directory_name.txt, and finally complete with nfo.txt\n\nIt allows to specify more easily default options for an entire set of video, directory, playlist and so on.\n\n## Features\n\n- [x] Youtube upload\n- [x] Peertube upload\n- Support of videos parameters (description, tags, category, licence, ...)\n  - [x] description\n  - [x] tags (no more than 30 characters per tag as Peertube does not support it)\n  - [x] categories\n  - [x] license: cca or not (Youtube only as Peertube uses Attribution by design)\n  - [x] privacy (between public, unlisted or private)\n  - [x] enabling/disabling comment (Peertube only as Youtube API does not support it)\n  - [x] nsfw (Peertube only as Youtube API does not support it)\n  - [x] set default language\n  - [x] thumbnail\n  - [x] multiple lines description (see [issue 4](https://git.lecygnenoir.info/LecygneNoir/prismedia/issues/4))\n  - [x] add videos to playlist\n  - [x] create playlist\n  - [x] schedule your video with publishAt\n  - [x] combine channel and playlist (Peertube only as channel is Peertube feature). See [issue 40](https://git.lecygnenoir.info/LecygneNoir/prismedia/issues/40) for detailed usage.\n- [x] Use a config file (NFO) file to retrieve videos arguments\n- [x] Allow to choose peertube or youtube upload (to resume failed upload for example)\n- [x] Usable on Desktop (Linux and/or Windows and/or MacOS)\n- [x] Different schedules on platforms to prepare preview\n- [ ] A usable graphical interface\n\n## Compatibility\n\n - If you still use python2, use the version 0.7.1 (no more updated)\n - If you use peertube before 1.0.0-beta4, use the version inside tag 1.0.0-beta3\n\n## Sources\ninspired by [peeror](https://git.rigelk.eu/rigelk/peeror) and [youtube-upload](https://github.com/tokland/youtube-upload)\n\n## Contributors\nThanks to: @Zykino, @meewan, @rigelk 😘',
    'author': 'LecygneNoir',
    'author_email': 'git@lecygnenoir.info',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://git.lecygnenoir.info/LecygneNoir/prismedia',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.5',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# coding: utf-8

"""
prismedia - tool to upload videos to Peertube and Youtube

Usage:
  prismedia --file=<FILE> [options]
  prismedia -f <FILE> --tags=STRING [options]
  prismedia -h | --help
  prismedia --version

Options:
  -f, --file=STRING Path to the video file to upload in mp4
  --name=NAME  Name of the video to upload. (default to video filename)
  --debug  Trigger some debug information like options used (default: no)
  -d, --description=STRING  Description of the video. (default: default description)
  -t, --tags=STRING  Tags for the video. comma separated.
                     WARN: tags with punctuation (!, ', ", ?, ...)
                           are not supported by Mastodon to be published from Peertube
  -c, --category=STRING  Category for the videos, see below. (default: Films)
  --cca  License should be CreativeCommon Attribution (affects Youtube upload only)
  -p, --privacy=STRING  Choose between public, unlisted or private. (default: private)
  --disable-comments  Disable comments (Peertube only as YT API does not support) (default: comments are enabled)
  --nsfw  Set the video as No Safe For Work (Peertube only as YT API does not support) (default: video is safe)
  --nfo=STRING  Configure a specific nfo file to set options for the video.
                By default Prismedia search a .txt based on the video name and will
                decode the file as UTF-8 (so make sure your nfo file is UTF-8 encoded)
                See nfo_example.txt for more details
  --platform=STRING  List of platform(s) to upload to, comma separated.
                     Supported platforms are youtube and peertube (default is both)
  --language=STRING  Specify the default language for video. See below for supported language. (default is English)
  --publishAt=DATE  Publish the video at the given DATE using local server timezone.
                    DATE should be on the form YYYY-MM-DDThh:mm:ss eg: 2018-03-12T19:00:00
                    DATE should be in the future
  --peertubeAt=DATE
  --youtubeAt=DATE  Override publishAt for the corresponding platform. Allow to create preview on specific platform
  --thumbnail=STRING    Path to a file to use as a thumbnail for the video.
                        Supported types are jpg and jpeg.
                        By default, prismedia search for an image based on video name followed by .jpg or .jpeg
  --channel=STRING Set the channel to use for the video (Peertube only)
                    If the channel is not found, spawn an error except if --channelCreate is set.
  --channelCreate  Create the channel if not exists. (Peertube only, default do not create)
                   Only relevant if --channel is set.
  --playlist=STRING Set the playlist to use for the video.
                    If the playlist is not found, spawn an error except if --playlistCreate is set.
  --playlistCreate  Create the playlist if not exists. (default do not create)
                    Only relevant if --playlist is set.
  -h --help  Show this help.
  --version  Show version.

Categories:
  Category is the type of video you upload. Default is films.
  Here are available categories from Peertube and Youtube:
    music, films, vehicles,
    sports, travels, gaming, people,
    comedy, entertainment, news,
    how to, education, activism, science & technology,
    science, technology, animals

Languages:
  Language of the video (audio track), choose one. Default is English
  Here are available languages from Peertube and Youtube:
    Arabic, English, French, German, Hindi, Italian,
    Japanese, Korean, Mandarin, Portuguese, Punjabi, Russian, Spanish

"""
import sys
if sys.version_info[0] < 3:
    raise Exception("Python 3 or a more recent version is required.")

import datetime
import logging
logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)

from docopt import docopt

from . import yt_upload
from . import pt_upload
from . import utils

try:
    # noinspection PyUnresolvedReferences
    from schema import Schema, And, Or, Optional, SchemaError
except ImportError:
    logging.error('This program requires that the `schema` data-validation library'
                  ' is installed: \n'
                  'see https://github.com/halst/schema\n')
    exit(1)
try:
    # noinspection PyUnresolvedReferences
    import magic
except ImportError:
    logging.error('This program requires that the `python-magic` library'
                  ' is installed, NOT the Python bindings to libmagic API \n'
                  'see https://github.com/ahupp/python-magic\n')
    exit(1)

VERSION = "prismedia v0.9.1"

VALID_PRIVACY_STATUSES = ('public', 'private', 'unlisted')
VALID_CATEGORIES = (
    "music", "films", "vehicles",
    "sports", "travels", "gaming", "people",
    "comedy", "entertainment", "news",
    "how to", "education", "activism", "science & technology",
    "science", "technology", "animals"
)
VALID_PLATFORM = ('youtube', 'peertube', 'none')
VALID_LANGUAGES = ('arabic', 'english', 'french',
                   'german', 'hindi', 'italian',
                   'japanese', 'korean', 'mandarin',
                   'portuguese', 'punjabi', 'russian', 'spanish')


def validateVideo(path):
    supported_types = ['video/mp4']
    detected_type = magic.from_file(path, mime=True)
    if detected_type not in supported_types:
        print("File", path, "detected type is", detected_type, "which is not one of", supported_types)

        force_file = ['y', 'yes']
        is_forcing = input("Are you sure you selected the correct file? (y/N)")
        if is_forcing.lower() not in force_file:
            return False

    return path


def validateCategory(category):
    if category.lower() in VALID_CATEGORIES:
        return True
    else:
        return False


def validatePrivacy(privacy):
    if privacy.lower() in VALID_PRIVACY_STATUSES:
        return True
    else:
        return False


def validatePlatform(platform):
    for plfrm in platform.split(','):
        if plfrm.lower().replace(" ", "") not in VALID_PLATFORM:
            return False

    return True


def validateLanguage(language):
    if language.lower() in VALID_LANGUAGES:
        return True
    else:
        return False


def validatePublish(publish):
    # Check date format and if date is future
    try:
        now = datetime.datetime.now()
        publishAt = datetime.datetime.strptime(publish, '%Y-%m-%dT%H:%M:%S')
        if now >= publishAt:
            return False
    except ValueError:
        return False
    return True


def validateThumbnail(thumbnail):
    supported_types = ['image/jpg', 'image/jpeg']
    if magic.from_file(thumbnail, mime=True) in supported_types:
        return thumbnail
    else:
        return False


def main():
    options = docopt(__doc__, version=VERSION)

    schema = Schema({
        '--file': And(str, validateVideo, error='file is not supported, please use mp4'),
        Optional('--name'): Or(None, And(
                                str,
                                lambda x: not x.isdigit(),
                                error="The video name should be a string")
                               ),
        Optional('--description'): Or(None, And(
                                        str,
                                        lambda x: not x.isdigit(),
                                        error="The video description should be a string")
                                      ),
        Optional('--tags'): Or(None, And(
                                    str,
                                    lambda x: not x.isdigit(),
                                    error="Tags should be a string")
                               ),
        Optional('--category'): Or(None, And(
                                    str,
                                    validateCategory,
                                    error="Category not recognized, please see --help")
                                   ),
        Optional('--language'): Or(None, And(
                                    str,
                                    validateLanguage,
                                    error="Language not recognized, please see --help")
                                   ),
        Optional('--privacy'): Or(None, And(
                                    str,
                                    validatePrivacy,
                                    error="Please use recognized privacy between public, unlisted or private")
                                  ),
        Optional('--nfo'): Or(None, str),
        Optional('--platform'): Or(None, And(str, validatePlatform, error="Sorry, upload platform not supported")),
        Optional('--publishAt'): Or(None, And(
                                    str,
                                    validatePublish,
                                    error="DATE should be the form YYYY-MM-DDThh:mm:ss and has to be in the future")
                                    ),
        Optional('--peertubeAt'): Or(None, And(
                                    str,
                                    validatePublish,
                                    error="DATE should be the form YYYY-MM-DDThh:mm:ss and has to be in the future")
                                    ),
        Optional('--youtubeAt'): Or(None, And(
                                    str,
                                    validatePublish,
                                    error="DATE should be the form YYYY-MM-DDThh:mm:ss and has to be in the future")
                                    ),
        Optional('--debug'): bool,
        Optional('--cca'): bool,
        Optional('--disable-comments'): bool,
        Optional('--nsfw'): bool,
        Optional('--thumbnail'): Or(None, And(
                                    str, validateThumbnail, error='thumbnail is not supported, please use jpg/jpeg'),
                                    ),
        Optional('--channel'): Or(None, str),
        Optional('--channelCreate'): bool,
        Optional('--playlist'): Or(None, str),
        Optional('--playlistCreate'): bool,
        '--help': bool,
        '--version': bool
    })

    options = utils.parseNFO(options)

    if not options.get('--thumbnail'):
        options = utils.searchThumbnail(options)

    try:
        options = schema.validate(options)
    except SchemaError as e:
        exit(e)

    if options.get('--debug'):
        print(sys.version)
        print(options)

    if options.get('--platform') is None or "peertube" in options.get('--platform'):
        pt_upload.run(options)
    if options.get('--platform') is None or "youtube" in options.get('--platform'):
        yt_upload.run(options)


if __name__ == '__main__':
    import warnings
    warnings.warn("use 'python -m prismedia', not 'python -m prismedia.upload'", DeprecationWarning)
    main()



#!/usr/bin/env python
# coding: utf-8
# From Youtube samples : https://raw.githubusercontent.com/youtube/api-samples/master/python/upload_video.py  # noqa

import http.client
import httplib2
import random
import time
import copy
import json
from os.path import splitext, basename, exists
import os
import google.oauth2.credentials
import datetime
import pytz
import logging
from tzlocal import get_localzone

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload
from google_auth_oauthlib.flow import InstalledAppFlow


from . import utils

logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)


# Explicitly tell the underlying HTTP transport library not to retry, since
# we are handling retry logic ourselves.
httplib2.RETRIES = 1

# Maximum number of times to retry before giving up.
MAX_RETRIES = 10

# Youtube retriables cases
RETRIABLE_EXCEPTIONS = (
    IOError,
    httplib2.HttpLib2Error,
    http.client.NotConnected,
    http.client.IncompleteRead,
    http.client.ImproperConnectionState,
    http.client.CannotSendRequest,
    http.client.CannotSendHeader,
    http.client.ResponseNotReady,
    http.client.BadStatusLine,
)

RETRIABLE_STATUS_CODES = [500, 502, 503, 504]


CLIENT_SECRETS_FILE = 'youtube_secret.json'
CREDENTIALS_PATH = ".youtube_credentials.json"
SCOPES = ['https://www.googleapis.com/auth/youtube.upload', 'https://www.googleapis.com/auth/youtube.force-ssl']
API_SERVICE_NAME = 'youtube'
API_VERSION = 'v3'


# Authorize the request and store authorization credentials.
def get_authenticated_service():
    check_authenticated_scopes()
    flow = InstalledAppFlow.from_client_secrets_file(
        CLIENT_SECRETS_FILE, SCOPES)
    if exists(CREDENTIALS_PATH):
        with open(CREDENTIALS_PATH, 'r') as f:
            credential_params = json.load(f)
            credentials = google.oauth2.credentials.Credentials(
                credential_params["token"],
                refresh_token=credential_params["_refresh_token"],
                token_uri=credential_params["_token_uri"],
                client_id=credential_params["_client_id"],
                client_secret=credential_params["_client_secret"]
            )
    else:
        credentials = flow.run_console()
        with open(CREDENTIALS_PATH, 'w') as f:
            p = copy.deepcopy(vars(credentials))
            del p["expiry"]
            json.dump(p, f)
    return build(API_SERVICE_NAME, API_VERSION, credentials=credentials,  cache_discovery=False)


def check_authenticated_scopes():
    if exists(CREDENTIALS_PATH):
        with open(CREDENTIALS_PATH, 'r') as f:
            credential_params = json.load(f)
            # Check if all scopes are present
            if credential_params["_scopes"] != SCOPES:
                logging.warning("Youtube: Credentials are obsolete, need to re-authenticate.")
                os.remove(CREDENTIALS_PATH)


def initialize_upload(youtube, options):
    path = options.get('--file')
    tags = None
    if options.get('--tags'):
        tags = options.get('--tags').split(',')

    category = None
    if options.get('--category'):
        category = utils.getCategory(options.get('--category'), 'youtube')

    language = None
    if options.get('--language'):
        language = utils.getLanguage(options.get('--language'), "youtube")

    license = None
    if options.get('--cca'):
        license = "creativeCommon"

    body = {
        "snippet": {
            "title": options.get('--name') or splitext(basename(path))[0],
            "description": options.get('--description') or "default description",
            "tags": tags,
            # if no category, set default to 1 (Films)
            "categoryId": str(category or 1),
            "defaultAudioLanguage": str(language or 'en')
        },
        "status": {
            "privacyStatus": str(options.get('--privacy') or "private"),
            "license": str(license or "youtube"),
        }
    }

    # If peertubeAt exists, use instead of publishAt
    if options.get('--youtubeAt'):
        publishAt = options.get('--youtubeAt')
    elif options.get('--publishAt'):
        publishAt = options.get('--publishAt')

    if 'publishAt' in locals():
        # Youtube needs microsecond and the local timezone from ISO 8601
        publishAt = publishAt + ".000001"
        publishAt = datetime.datetime.strptime(publishAt, '%Y-%m-%dT%H:%M:%S.%f')
        tz = get_localzone()
        tz = pytz.timezone(str(tz))
        publishAt = tz.localize(publishAt).isoformat()
        body['status']['publishAt'] = str(publishAt)

    if options.get('--playlist'):
        playlist_id = get_playlist_by_name(youtube, options.get('--playlist'))
        if not playlist_id and options.get('--playlistCreate'):
            playlist_id = create_playlist(youtube, options.get('--playlist'))
        elif not playlist_id:
            logging.warning("Youtube: Playlist `" + options.get('--playlist') + "` is unknown.")
            logging.warning("If you want to create it, set the --playlistCreate option.")
            playlist_id = ""
    else:
        playlist_id = ""

    # Call the API's videos.insert method to create and upload the video.
    insert_request = youtube.videos().insert(
        part=','.join(list(body.keys())),
        body=body,
        media_body=MediaFileUpload(path, chunksize=-1, resumable=True)
    )
    video_id = resumable_upload(insert_request, 'video', 'insert')

    # If we get a video_id, upload is successful and we are able to set thumbnail
    if video_id and options.get('--thumbnail'):
        set_thumbnail(youtube, options.get('--thumbnail'), videoId=video_id)

    # If we get a video_id and a playlist_id, upload is successful and we are able to set playlist
    if video_id and playlist_id != "":
        set_playlist(youtube, playlist_id, video_id)


def get_playlist_by_name(youtube, playlist_name):
    response = youtube.playlists().list(
        part='snippet,id',
        mine=True,
        maxResults=50
    ).execute()
    for playlist in response["items"]:
        if playlist["snippet"]['title'] == playlist_name:
            return playlist['id']


def create_playlist(youtube, playlist_name):
    template = ('Youtube: Playlist %s does not exist, creating it.')
    logging.info(template % (str(playlist_name)))
    resources = build_resource({'snippet.title': playlist_name,
                                'snippet.description': '',
                                'status.privacyStatus': 'public'})
    response = youtube.playlists().insert(
        body=resources,
        part='status,snippet,id'
    ).execute()
    return response["id"]


def build_resource(properties):
    resource = {}
    for p in properties:
        # Given a key like "snippet.title", split into "snippet" and "title", where
        # "snippet" will be an object and "title" will be a property in that object.
        prop_array = p.split('.')
        ref = resource
        for pa in range(0, len(prop_array)):
            is_array = False
            key = prop_array[pa]

            # For properties that have array values, convert a name like
            # "snippet.tags[]" to snippet.tags, and set a flag to handle
            # the value as an array.
            if key[-2:] == '[]':
                key = key[0:len(key)-2:]
                is_array = True

            if pa == (len(prop_array) - 1):
                # Leave properties without values out of inserted resource.
                if properties[p]:
                    if is_array:
                        ref[key] = properties[p].split(',')
                    else:
                        ref[key] = properties[p]
            elif key not in ref:
                # For example, the property is "snippet.title", but the resource does
                # not yet have a "snippet" object. Create the snippet object here.
                # Setting "ref = ref[key]" means that in the next time through the
                # "for pa in range ..." loop, we will be setting a property in the
                # resource's "snippet" object.
                ref[key] = {}
                ref = ref[key]
            else:
                # For example, the property is "snippet.description", and the resource
                # already has a "snippet" object.
                ref = ref[key]
    return resource


def set_thumbnail(youtube, media_file, **kwargs):
    kwargs = utils.remove_empty_kwargs(**kwargs)
    request = youtube.thumbnails().set(
        media_body=MediaFileUpload(media_file, chunksize=-1,
                                   resumable=True),
        **kwargs
    )

    # See full sample for function
    return resumable_upload(request, 'thumbnail', 'set')


def set_playlist(youtube, playlist_id, video_id):
    logging.info('Youtube: Configuring playlist...')
    resource = build_resource({'snippet.playlistId': playlist_id,
                               'snippet.resourceId.kind': 'youtube#video',
                               'snippet.resourceId.videoId': video_id,
                               'snippet.position': ''}
                              )
    try:
        youtube.playlistItems().insert(
            body=resource,
            part='snippet'
        ).execute()
    except Exception as e:
        if hasattr(e, 'message'):
            logging.error("Youtube: Error: " + str(e.message))
            exit(1)
        else:
            logging.error("Youtube: Error: " + str(e))
            exit(1)
    logging.info('Youtube: Video is correctly added to the playlist.')


# This method implements an exponential backoff strategy to resume a
# failed upload.
def resumable_upload(request, resource, method):
    response = None
    error = None
    retry = 0
    while response is None:
        try:
            template = 'Youtube: Uploading %s...'
            logging.info(template % resource)
            status, response = request.next_chunk()
            if response is not None:
                if method == 'insert' and 'id' in response:
                    logging.info('Youtube : Video was successfully uploaded.')
                    template = 'Youtube: Watch it at https://youtu.be/%s (post-encoding could take some time)'
                    logging.info(template % response['id'])
                    return response['id']
                elif method != 'insert' or "id" not in response:
                    logging.info('Youtube: Thumbnail was successfully set.')
                else:
                    template = ('Youtube : The upload failed with an '
                                'unexpected response: %s')
                    logging.error(template % response)
                    exit(1)
        except HttpError as e:
            if e.resp.status in RETRIABLE_STATUS_CODES:
                template = 'Youtube : A retriable HTTP error %d occurred:\n%s'
                error = template % (e.resp.status, e.content)
            else:
                raise
        except RETRIABLE_EXCEPTIONS as e:
            error = 'Youtube : A retriable error occurred: %s' % e

    if error is not None:
        logging.warning(error)
        retry += 1
        if retry > MAX_RETRIES:
            logging.error('Youtube : No longer attempting to retry.')
            exit(1)

        max_sleep = 2 ** retry
        sleep_seconds = random.random() * max_sleep
        logging.warning('Youtube : Sleeping %f seconds and then retrying...'
              % sleep_seconds)
        time.sleep(sleep_seconds)


def run(options):
    youtube = get_authenticated_service()
    try:
        initialize_upload(youtube, options)
    except HttpError as e:
        logging.error('Youtube : An HTTP error %d occurred:\n%s' % (e.resp.status,
                                                            e.content))
